import { OnApplicationBootstrap } from '@nestjs/common';
import { TypeOrmModule } from '@nestjs/typeorm';

import { ModuleRef } from '@nestjs/core';
import { EventBus, Logger, PluginCommonModule, VendurePlugin, Injector } from '@vendure/core';

import { adminApiExtensions } from './api/api-extensions';
import { ContactsAdminResolver } from './api/contacts-admin.resolver';

import { Contacts } from './entities/contacts.entity';
import { ContactSubmittedEvent } from './events/contact-submitted.event';
import { ContactsService } from './services/contacts.service';
import { AdminService } from './services/admin-email.service';
import { autoReplyHandler, contactFormHandler, initializeAdminService } from './handler/contacts.handler';

@VendurePlugin({
  imports: [PluginCommonModule, TypeOrmModule.forFeature([Contacts])],
  providers: [ContactsService, ContactsAdminResolver, AdminService],
  compatibility: '^2.0.0',
  entities: [Contacts],
  adminApiExtensions: {
    schema: adminApiExtensions,
    resolvers: [ContactsAdminResolver]
  },
  shopApiExtensions: {
    schema: adminApiExtensions,
    resolvers: [ContactsAdminResolver]
  }
})
export class ContactsPlugin implements OnApplicationBootstrap {
  constructor(
    private eventBus: EventBus,
    private adminService: AdminService,
    private moduleRef: ModuleRef
  ) {}

  async onApplicationBootstrap() {
    // Initialize the adminService in the handler
    await initializeAdminService(this.adminService);

    // Register the event handlers
    this.eventBus.ofType(ContactSubmittedEvent).subscribe(async (event) => {
      const globals = { fromAddress: 'noreply@example.com' };

      try {
        const contactFormResult = await contactFormHandler.handle(event, globals, this.moduleRef as any);
        // Logger.info(`Contact form handler result: ${JSON.stringify(contactFormResult, null, 2)}`, 'ContactsPlugin');
      } catch (error) {
        Logger.error(`Error in contact form handler: ${(error as Error).message}`, 'ContactsPlugin');
      }

      try {
        const autoReplyResult = await autoReplyHandler.handle(event, globals, this.moduleRef as any);
        // Logger.info(`Auto-reply handler result: ${JSON.stringify(autoReplyResult, null, 2)}`, 'ContactsPlugin');
      } catch (error) {
        Logger.error(`Error in auto-reply handler: ${(error as Error).message}`, 'ContactsPlugin');
      }
      Logger.info(`Handling ContactSubmittedEvent: ${JSON.stringify(event.input, null, 2)}`, 'ContactsPlugin');
    });
  }
}
