import { OnApplicationBootstrap } from '@nestjs/common';
import { TypeOrmModule } from '@nestjs/typeorm';

import { ModuleRef } from '@nestjs/core';
import { EventBus, Logger, PluginCommonModule, Type, VendurePlugin } from '@vendure/core';

import { adminApiExtensions } from './api/api-extensions';
import { ProductContactAdminResolver } from './api/product-contact-admin.resolver';

import { ProductContact } from './entities/product-contact.entity';
import { ProductContactSubmittedEvent } from './events/product-contact-submitted.event';
import { ProductContactService } from './services/product-contact.service';
import { AdminService } from '../EMAILS/store-contact/services/admin-email.service';
import { productContactAutoReplyHandler, productContactFormHandler, initializeAdminService } from './handler/product-contact.handler';

@VendurePlugin({
    imports: [PluginCommonModule, TypeOrmModule.forFeature([ProductContact])],
    providers: [ProductContactService, ProductContactAdminResolver, AdminService],
    compatibility: '^2.0.0',
    entities: [ProductContact],
    adminApiExtensions: {
        schema: adminApiExtensions,
        resolvers: [ProductContactAdminResolver]
    },
    shopApiExtensions: {
        schema: adminApiExtensions,
        resolvers: [ProductContactAdminResolver]
    },
})
export class ProductContactPlugin {
    constructor(
        private eventBus: EventBus,
        private adminService: AdminService,
        private moduleRef: ModuleRef
    ) {}

    async onApplicationBootstrap() {
        // Initialize the adminService in the handler
        await initializeAdminService(this.adminService);
    
        // Register the event handlers
        this.eventBus.ofType(ProductContactSubmittedEvent).subscribe(async (event) => {
          const globals = { fromAddress: 'noreply@example.com' };
    
          try {
            const productContactFormResult = await productContactFormHandler.handle(event, globals, this.moduleRef as any);
            // Logger.info(`Contact form handler result: ${JSON.stringify(productContactFormResult, null, 2)}`, 'ContactsPlugin');
          } catch (error) {
            Logger.error(`Error in contact form handler: ${(error as Error).message}`, 'ContactsPlugin');
          }
    
          try {
            const productContactAutoReplyResult = await productContactAutoReplyHandler.handle(event, globals, this.moduleRef as any);
            // Logger.info(`Auto-reply handler result: ${JSON.stringify(productContactAutoReplyResult, null, 2)}`, 'ContactsPlugin');
          } catch (error) {
            Logger.error(`Error in auto-reply handler: ${(error as Error).message}`, 'ContactsPlugin');
          }
          Logger.info(`Handling ContactSubmittedEvent: ${JSON.stringify(event.input, null, 2)}`, 'ContactsPlugin');
        });
      }

}
