import { Inject, Injectable } from '@nestjs/common';
import { EventBus, ID, Logger, ProductVariant, RequestContext, TransactionalConnection } from '@vendure/core';

import { ProductContact } from '../entities/product-contact.entity';
import { ProductContactSubmittedEvent } from '../events/product-contact-submitted.event';
import { ProductContactFormInput } from '../types';

@Injectable()
export class ProductContactService {
    constructor(
        private connection: TransactionalConnection,
        private eventBus: EventBus
    ) {}
    
    async submitProductContactForm(ctx: RequestContext, input: ProductContactFormInput): Promise<ProductContact> {
        const productVariant = await this.connection.getRepository(ctx, ProductVariant).findOne({
            relations: ['translations'],
            where: {
                translations: {
                    name: input.productVariantName
                }
            }
        });
        if (!productVariant) {
            throw new Error(`ProductVariant with name ${input.productVariantName} not found`);
        }
        
        const productContactRepository = this.connection.rawConnection.getRepository(ProductContact);
        const productContact = await productContactRepository.save(new ProductContact({
            productVariant,
            firstName: input.firstName,
            lastName: input.lastName,
            email: input.email,
            reason: input.reason,
            message: input.message,
        }));

        Logger.info(`Product contact form saved: ${JSON.stringify(productContact, null, 2)}`, 'ProductContactService');
        
        await this.eventBus.publish(new ProductContactSubmittedEvent(ctx, input));
        return productContact;
    }

    async findOne(id: ID): Promise<ProductContact | null> {
        Logger.info(`Finding product contact with ID: ${id}`, 'ProductContactService');
        
        const productContactRepository = this.connection.rawConnection.getRepository(ProductContact);
        const productContact = await productContactRepository.findOne({ where: { id } });
        
        Logger.info(`Product contact found: ${JSON.stringify(productContact, null, 2)}`, 'ProductContactService');
        
        return productContact;
    }
}
