import {
    dummyPaymentHandler,
    DefaultJobQueuePlugin,
    DefaultSearchPlugin,
    VendureConfig,
    DefaultGuestCheckoutStrategy,
    LanguageCode,
} from '@vendure/core';
import { defaultEmailHandlers, EmailPlugin, EmailPluginOptions } from '@vendure/email-plugin';
import { AssetServerPlugin } from '@vendure/asset-server-plugin';
import { AdminUiPlugin } from '@vendure/admin-ui-plugin';
import 'dotenv/config';
import path from 'path';

import { userOrderConfirmationHandler } from './plugins/EMAILS/default-email-plugin/handler/order-confirmation.handler';
import { userEmailVerificationHandler } from './plugins/EMAILS/default-email-plugin/handler/email-verification.handler';
import { userPasswordResetHandler } from './plugins/EMAILS/default-email-plugin/handler/password-reset.handler';
import { autoReplyHandler, contactFormHandler } from './plugins/EMAILS/store-contact/handler/contacts.handler';
import { ContactsPlugin } from './plugins/EMAILS/store-contact/contacts.plugin';
import { BobgoPlugin } from './plugins/bobgo-shipping/bobgo.plugin';
import netcashPaymentHandler from './plugins/netcash-payment/netcash-payment.handler';
import { NetcashPlugin } from './plugins/netcash-payment/netcash.plugin';
import { ProductContactPlugin } from './plugins/product-contact/product-contact.plugin';
import { productContactAutoReplyHandler, productContactFormHandler } from './plugins/product-contact/handler/product-contact.handler';
import { WebhookPlugin } from './plugins/webhook/webhook.plugin';
import { BobgoWebhookPlugin } from './plugins/bobgo-webhook/bobgo-webhook.plugin';
import { RestPlugin } from './plugins/rest/rest.plugin';

const IS_DEV = process.env.APP_ENV === 'dev';

const smtp_username = process.env.SMTP_USERNAME;
const smtp_password = process.env.SMTP_PASSWORD;

export const config: VendureConfig = {
  apiOptions: {
    port: 3000,
    adminApiPath: 'admin-api',
    shopApiPath: 'shop-api',
    // The following options are useful in development mode,
    // but are best turned off for production for security
    // reasons.
    ...(IS_DEV ? {
      adminApiPlayground: {
        settings: { 'request.credentials': 'include' },
      },
      adminApiDebug: true,
      shopApiPlayground: {
        settings: { 'request.credentials': 'include' },
      },
      shopApiDebug: true,
    } : {}),
  },
  authOptions: {
    tokenMethod: ['bearer', 'cookie'],
    superadminCredentials: {
      identifier: process.env.SUPERADMIN_USERNAME,
      password: process.env.SUPERADMIN_PASSWORD,
    },
    cookieOptions: {
      secret: process.env.COOKIE_SECRET,
    },
    
    // to verify email address upon registration
    requireVerification: true,
  },
  dbConnectionOptions: {
    type: 'mariadb',
    // See the README.md "Migrations" section for an explanation of
    // the `synchronize` and `migrations` options.
    synchronize: false,
    migrations: [path.join(__dirname, './migrations/*.+(js|ts)')],
    logging: false,
    database: process.env.DB_NAME,
    host: process.env.DB_HOST,
    port: +process.env.DB_PORT,
    username: process.env.DB_USERNAME,
    password: process.env.DB_PASSWORD,
  },
  paymentOptions: {
    paymentMethodHandlers: [
      netcashPaymentHandler,
      dummyPaymentHandler
    ],
  },
  // When adding or altering custom field definitions, the database will
  // need to be updated. See the "Migrations" section in README.md.

  // Adding custom fields to built-in entities
  customFields: {
    ProductVariant: [
      { name: 'unitWeight', type: 'float',
        label: [{ languageCode: LanguageCode.en, value: 'Unit Weight (kg)' }]
      },
      { name: 'unitLength', type: 'float',
        label: [{ languageCode: LanguageCode.en, value: 'Unit Length (cm)' }]
      },
      { name: 'unitWidth', type: 'float',
        label: [{ languageCode: LanguageCode.en, value: 'Unit Width (cm)' }]
      },
      { name: 'unitHeight', type: 'float',
        label: [{ languageCode: LanguageCode.en, value: 'Unit Height (cm)' }]
      }
    ]
  },

  // Allowing guest checkout
  orderOptions: {
    guestCheckoutStrategy: new DefaultGuestCheckoutStrategy({
      allowGuestCheckouts: true,
      allowGuestCheckoutForRegisteredCustomers: false
    })
  },

  plugins: [
    // to bulk update product variants
    // Reference: https://pinelab-plugins.com/plugin/vendure-plugin-variant-bulk-update/
    // VariantBulkUpdatePlugin,

    // shipping plugin
    BobgoPlugin,

    // netcash plugin
    NetcashPlugin, 

    AssetServerPlugin.init({
      route: 'assets',
      assetUploadDir: path.join(__dirname, '../static/assets'),
      // For local dev, the correct value for assetUrlPrefix should
      // be guessed correctly, but for production it will usually need
      // to be set manually to match your production url.
      assetUrlPrefix: IS_DEV ? undefined : 'https://www.my-shop.com/assets/',
    }),
    DefaultJobQueuePlugin.init({ useDatabaseForBuffer: true }),
    DefaultSearchPlugin.init({ bufferUpdates: false, indexStockStatus: true }),

    // BUILT-IN EMAIL PLUGIN
    EmailPlugin.init({
      transport: {
        type: 'smtp',
        host: 'smtp.gmail.com', // SMTP server
        port: 587,
        auth: {
          user: smtp_username, // SMTP username
          pass: smtp_password // SMTP password - 16-character google app password
        },
        secure: false, // false for TLS (port 587)
        logging: true, // Optional: for troubleshooting
        debug: true // Optional: for troubleshooting
      },
      devMode: false,
      outputPath: path.join(__dirname, '../static/email/test-emails'),
      route: 'mailbox',
      handlers: [
        // Default email handlers
        ...defaultEmailHandlers,
        userEmailVerificationHandler,
        userOrderConfirmationHandler,
        userPasswordResetHandler,
        
        // Store Contact Forms
        contactFormHandler,
        autoReplyHandler,

        // Product Contact Forms
        productContactFormHandler,
        productContactAutoReplyHandler,
      ],
      templatePath: path.join(__dirname, '../static/email/templates'),
      globalTemplateVars: {
        // storefront implementation:
        // -> Local Storefront running at http://localhost:4200
        // -> LIVE storefront is running at: http://157.245.36.128:5200
        fromAddress: '"DAN Southern Africa" <noreply@example.com>',
        verifyEmailAddressUrl: 'http://localhost:4200/verify-account',
        passwordResetUrl: 'http://localhost:4200/reset-new-password',
        changeEmailAddressUrl: 'http://localhost:4200/verify-email-address-change'
      },
    } as EmailPluginOptions),

    // PLUGINS USING DEFAULT EMAIL SETTINGS 
    ContactsPlugin,
    ProductContactPlugin,

    AdminUiPlugin.init({
      route: 'admin',
      port: 3002,
      adminUiConfig: {
        apiPort: 3000,
      },
    }),

    // Webhook endpoint 
    WebhookPlugin,
    
    // to suscribe to Bobgo webhook events
    BobgoWebhookPlugin,
      RestPlugin,
],
};
